//[skeleton0
package auditory.described;

import java.util.*;
import javax.sound.midi.*;

import event.*;

/**
 * An encapsulation of a music score
 *
 * @author  Prof. David Bernstein, James Madison University
 * @see     "The Design and Implementation of Multimedia Software  2011"
 * @version 1.0
 */
public class Score
{
//]skeleton0
//[skeleton1
    private HashMap<Part, MidiChannel>         channelMap;
    private HashMap<Part, String>              parts;
    private int                                timeSignatureDenominator, 
                                               timeSignatureNumerator,
                                               millisPerMeasure;
//]skeleton1

//[constructor
    /**
     * Default Constructor
     */
    public Score()
    {
       channelMap = new HashMap<Part, MidiChannel>();
       parts      = new HashMap<Part, String>();       
    }
//]constructor

//[skeleton2
    /**
     * Add a Part to this Score
     *
     * @param part   The Part to add
     */
    public void addPart(Part part, String instrument)
    {
       parts.put(part, instrument);
    }

    /**
     * Get the Part objects in this Score
     *
     * @return   An Iterator of the Part objects in this Score
     */
    public Iterator<Part> getParts()
    {
       return parts.keySet().iterator();       
    }

    /**
     * Get a String representation of the instrument
     * for a particular Part
     *
     * @param part   The Part of interest
     * @return       The name of the instrument 
     */
    public String getInstrumentName(Part part)
    {
       return parts.get(part);       
    }

    /**
     * Remove a Part from this Score
     *
     * @param part   The Part to remove
     */
    public void removePart(Part part)
    {
       parts.remove(part);
    }    
//]skeleton2

//[render
    /**
     * Render this Score
     */
    public void render()
    {
       Iterator<Part>              i;
       MidiChannel                 channel;
       Part                        part;
       
       i = parts.keySet().iterator();
       while (i.hasNext())
       {
          part       = i.next();
          channel    = channelMap.get(part);
          part.render(channel);          
       }
    }
//]render

//[setters    
    /**
     * Set the MidiChannel associated with a Part
     *
     * @param part     The Part
     * @param channel  The MidiChannel for that Part
     */
    public void setChannel(Part part, MidiChannel channel)
    {
       channelMap.put(part, channel);             
    }

    /**
     * Set the tempo for this Part
     *
     * @param millisPerMeasure  The tempo (in milliseconds per measure)
     */
    public void setTempo(int millisPerMeasure)
    {
       this.millisPerMeasure = millisPerMeasure;       
    }
    
    /**
     * Set the time signature for this Part
     *
     * @param numerator    The numerator of the time signature
     * @param denominator  The denominator of the time signature
     */
    public void setTimeSignature(int numerator, int denominator)
    {
       this.timeSignatureNumerator   = numerator;
       this.timeSignatureDenominator = denominator;
    }
//]setters

//[upbeat
    /**
     * Handle an upbeat message
     *
     * @param metronome   The Metronome that the Part objects listen to
     */
    public void upbeat(Metronome metronome)
    {
       Iterator<Part>              i;
       Part                        part;

       i = parts.keySet().iterator();
       while (i.hasNext())
       {
          part       = i.next();
          part.upbeat(metronome);
          part.setTimeSignature(timeSignatureNumerator, 
                                timeSignatureDenominator);
          part.setTempo(millisPerMeasure);
       }
    }
//]upbeat
//[skeleton3
}
//]skeleton3
