//[skeleton0
package visual.dynamic.described;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import javax.swing.*;

/**
 * A TweeningSprite is a Sprite that contains "key times" and the ability
 * to generate "in between times"
 *
 * @author  Prof. David Bernstein, James Madison University
 * @see     "The Design and Implementation of Multimedia Software  2011"
 * @version 1.0
 */
public abstract class TweeningSprite extends AbstractSprite
{
    protected ArrayList<Integer>  keyTimes;
    protected ArrayList<Point2D>  locations;
    protected ArrayList<Double>   rotations, scalings;
    private   double              frac;
    private   int                 currentIndex, endState, lastTime;
    private   int                 nextIndex, nextKT;
    
    public static final int REMAIN         = 0;
    public static final int REMOVE         = 1;

    /**
     * Default Constructor
     */
    public TweeningSprite()
    {       
       super();
       
       keyTimes  = new ArrayList<Integer>();
       locations = new ArrayList<Point2D>();
       rotations = new ArrayList<Double>();
       scalings  = new ArrayList<Double>();
       endState  = REMAIN;
       
       initialize();
    }
//]skeleton0

//[addKeyTime
    /**
     * Add a key time
     *
     * @param keyTime     The time
     * @param location    The location of the sprite at this time
     * @param rotation    The rotation of the sprite (null to align with path)
     * @param scaling     The scaling of the sprite at this time
     *
     * @return   The index of this key time (or -1 if the key time exists)
     */
    protected int addKeyTime(int keyTime, Point2D location,
                             Double rotation, Double scaling)
    {
       boolean     keepLooking;
       int         existingKT, i, index;

       existingKT  = -1;
       keepLooking = true;
       
       i = 0;
       while ((i < keyTimes.size()) && keepLooking)
       {
          existingKT = ((Integer)keyTimes.get(i)).intValue();
          
          if (existingKT >= keyTime) keepLooking = false;
          else                       i++;
       }

       if ((existingKT == i) && !keepLooking)  // Duplicate
       {
          i = -1;
       }
       else
       {
          keyTimes.add(i, new Integer(keyTime));
          locations.add(i, location);
          rotations.add(i, rotation);
          scalings.add(i, scaling);
       }
       
       return i;
    }
//]addKeyTime

//[handleTick
    /**
     * Handle a tick event (generated by the Stage)
     *
     * @param time  The current time (in milliseconds)
     */
    public void handleTick(int time)
    {
       boolean         aligned;
       int             currentKT, duration, finalKT, firstKT;

       // See if the Metronome has reset
       if (time < lastTime) reinitialize();

       // Get important key times
       finalKT   = keyTimes.get(keyTimes.size()-1).intValue();
       firstKT   = keyTimes.get(0).intValue();
       currentKT = keyTimes.get(currentIndex).intValue();
       nextKT    = keyTimes.get(nextIndex).intValue();

       // Handle end states
       if (time >= finalKT)
       {
          if (endState == REMOVE) setVisible(false);
          return;
       }

       // Record the last time
       lastTime  =  time;

       // Set the visibility
       if ((time < finalKT) && (time >= firstKT)) setVisible(true);
       
       // Update the important indexes and times if necessary
       if (time < nextKT)
       {
          nextIndex = currentIndex + 1;
          nextKT    = keyTimes.get(nextIndex).intValue();
       }
       else
       {
          while (time > nextKT)
          {
             currentIndex++;
             nextIndex++;             
             currentKT = keyTimes.get(currentIndex).intValue();
             nextKT    = keyTimes.get(nextIndex).intValue();
          }

          if (time >= finalKT)
          {
             nextIndex = currentIndex;
             nextKT    = keyTimes.get(nextIndex).intValue();
          }
       }

       // Determine the interpolation fraction
       if (nextKT == currentKT) frac = 1.0;
       else frac = (double)(time - currentKT)/(double)(nextKT - currentKT);

       // Tween the location, rotation and scale
       tweenLocation(currentIndex, nextIndex, frac);
       tweenScaling(currentIndex, nextIndex, frac);
       tweenRotation(currentIndex, nextIndex, frac);
    }
//]handleTick

    /**
     * Get the interpolation fraction
     *
     * @return  The interpolation fraction (in [0, 1])
     */
    protected double getInterpolationFraction()
    {
       return frac;
    }

    /**
     * Get the active key time index
     *
     * @return The index of the "most recent" key time
     */
    protected int getKeyTimeIndex()
    {
       return currentIndex;
    }

    /**
     * Get the next key time index
     *
     * @return The index of the "next" key time
     */
    protected int getNextKeyTimeIndex()
    {
       return nextIndex;
    }

    /**
     * Initialize state variables
     */
    protected void initialize()
    {
       currentIndex =  0;
       nextIndex    =  1;
       frac         =  0.0;
       lastTime     =  Integer.MAX_VALUE;
       setVisible(true);       
    }

    /**
     * Re-initialize state variables
     */
    protected void reinitialize()
    {
       initialize();
       super.reinitialize();
    }

    /**
     * Set the "end state" for this Sprite
     *
     * @param endState   Either REMOVE, REMAIN, or REPEAT
     */
    public void setEndState(int endState)
    {
       this.endState = REMOVE;
       if      (endState == REMAIN) this.endState = endState;
    }

//[tweenLocation
    /**
     * Determine the current 'tweened location
     * 
     * @param currentIndex   The index of the current key time
     * @param nextIndex      The index of the next key time
     * @param frac           The interpolation fraction
     */
    protected void tweenLocation(int currentIndex, int nextIndex, 
                                 double frac)
    {
       double          x, y;
       Point2D         currentKTLocation, nextKTLocation;
       
       currentKTLocation = locations.get(currentIndex);
       nextKTLocation    = locations.get(nextIndex);

       x = currentKTLocation.getX() + 
           frac*(nextKTLocation.getX()- currentKTLocation.getX());
       y = currentKTLocation.getY() + 
          frac*(nextKTLocation.getY() - currentKTLocation.getY());

       setLocation(x, y);
    }
//]tweenLocation

//[tweenRotation
    /**
     * Determine the current 'tweened rotation
     * 
     * @param currentIndex   The index of the current key time
     * @param nextIndex      The index of the next key time
     * @param frac           The interpolation fraction
     */
    protected void tweenRotation(int currentIndex, int nextIndex, 
                                 double frac)
    {
       double          currentKTRotation, nextKTRotation, r;
       Double          rotation;
       Point2D         currentKTLocation, nextKTLocation;

       rotation = rotations.get(currentIndex);

       if (rotation == null)  // aligned with the line segment
       {
//[alignedrotationtweening
          currentKTLocation = locations.get(currentIndex);
          nextKTLocation    = locations.get(nextIndex);

          r=Math.atan((nextKTLocation.getY()-currentKTLocation.getY())/
                      (nextKTLocation.getX()-currentKTLocation.getX()) );
//]alignedrotationtweening
       }
       else                  // pure rotation tweening
       {
//[purerotationtweening
          currentKTRotation = rotation.doubleValue();

          rotation = rotations.get(nextIndex);
          if (rotation == null) nextKTRotation = currentKTRotation;
          else                  nextKTRotation = rotation.doubleValue();
          
          r = currentKTRotation + frac*(nextKTRotation-currentKTRotation);
       }
//]purerotationtweening
       
       setRotation(r);
    }
//]tweenRotation

//[ScaleTween
    /**
     * Determine the current 'tweened scaling
     * 
     * @param currentIndex   The index of the current key time
     * @param nextIndex      The index of the next key time
     * @param frac           The interpolation fraction
     */
    protected void tweenScaling(int currentIndex, int nextIndex, double frac)
    {
       double          currentKTScaling, nextKTScaling, s;

       currentKTScaling = scalings.get(currentIndex).doubleValue();
       nextKTScaling    = scalings.get(nextIndex).doubleValue();
       s = currentKTScaling + frac*(nextKTScaling - currentKTScaling);
       setScale(s);
    }
//]tweenScale
//[skeleton1
}
//]skeleton1
